package bankgui;

import java.awt.Component;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Window;
import java.awt.Insets;
import java.awt.event.KeyEvent;

import javax.swing.JSplitPane;
import javax.swing.JLabel;
import javax.swing.SwingUtilities;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JButton;
import javax.swing.JTabbedPane;
import javax.swing.BorderFactory;
import javax.swing.border.TitledBorder;

import bank.ATM;
import bank.BankCard;
import bank.CentralHost;
import bank.Clock;


public class BankGui extends JFrame implements BankModelInteraction {

	private final Clock clock = Clock.getInstance();  //  @jve:decl-index=0:
	private final CentralHost host = CentralHost.getInstance();  //  @jve:decl-index=0:
	private final ATM atm;
	private CentralHostPanel centralHostPanel;
	private ATMPanel atmPanel;
	private AccountsPanel accountsPanel;
	private CardsPanel cardsPanel;

	private static final long serialVersionUID = 1L;
	private JPanel mainPane = null;
	private JSplitPane jSplitPane = null;
	private JPanel timePanel = null;
	private JLabel timeLabel = null;
	private JButton timeAdvanceButton = null;
	private JTabbedPane interactionPane = null;
	private JTabbedPane overviewPane = null;
	private JButton quitButton = null;
	private JLabel dateLabel = null;


	/**
	 * This method initializes jSplitPane	
	 * 	
	 * @return javax.swing.JSplitPane	
	 */
	private JSplitPane getJSplitPane() {
		if (jSplitPane == null) {
			jSplitPane = new JSplitPane();
			jSplitPane.setOrientation(JSplitPane.VERTICAL_SPLIT);
			jSplitPane.setOneTouchExpandable(true);
			jSplitPane.setResizeWeight(0);
			jSplitPane.setTopComponent(getInteractionPane());
			jSplitPane.setBottomComponent(getOverviewPane());
		}
		return jSplitPane;
	}

	/**
	 * This method initializes timePanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getTimePanel() {
		if (timePanel == null) {
			GridBagConstraints dateLabelConstraints = new GridBagConstraints();
			dateLabelConstraints.gridy = 0;
			dateLabelConstraints.anchor = GridBagConstraints.WEST;
			dateLabelConstraints.weightx = 1.0D;
			dateLabelConstraints.insets = new Insets(0, 15, 0, 0);
			dateLabelConstraints.gridx = 0;
			GridBagConstraints timeAdvanceButtonConstraints = new GridBagConstraints();
			timeAdvanceButtonConstraints.insets = new Insets(5, 5, 5, 15);
			timeAdvanceButtonConstraints.gridy = 0;
			timeAdvanceButtonConstraints.gridheight = 2;
			timeAdvanceButtonConstraints.gridx = 1;
			GridBagConstraints timeLabelConstraints = new GridBagConstraints();
			timeLabelConstraints.gridy = 1;
			timeLabelConstraints.weightx = 1.0D;
			timeLabelConstraints.anchor = GridBagConstraints.WEST;
			timeLabelConstraints.insets = new Insets(0, 15, 0, 0);
			timeLabelConstraints.gridx = 0;
			dateLabel = new JLabel();
			TitledBorder timePanelBorder = BorderFactory.createTitledBorder(null, "Time", TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.DEFAULT_POSITION, null, null);
			timeLabel = new JLabel();
			timePanel = new JPanel();
			timePanel.setLayout(new GridBagLayout());
			timePanel.setBorder(timePanelBorder);
			timePanel.add(timeLabel, timeLabelConstraints);
			timePanel.add(getTimeAdvanceButton(), timeAdvanceButtonConstraints);
			timePanel.add(dateLabel, dateLabelConstraints);
		}
		return timePanel;
	}

	/**
	 * This method initializes timeAdvanceButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getTimeAdvanceButton() {
		if (timeAdvanceButton == null) {
			timeAdvanceButton = new JButton();
			timeAdvanceButton.setText("Advance");
			timeAdvanceButton.setMnemonic(KeyEvent.VK_V);
			timeAdvanceButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					clock.tick();
					updateTimeView();
				}
			});
		}
		return timeAdvanceButton;
	}

	/**
	 * This method initializes interactionPane	
	 * 	
	 * @return javax.swing.JTabbedPane	
	 */
	private JTabbedPane getInteractionPane() {
		if (interactionPane == null) {
			interactionPane = new JTabbedPane();
			interactionPane.addTab("Central Host", null, centralHostPanel, null);
			interactionPane.setMnemonicAt(interactionPane.indexOfTab("Central Host"), KeyEvent.VK_H);
			interactionPane.addTab("ATM", null, atmPanel, null);
			interactionPane.setMnemonicAt(interactionPane.indexOfTab("ATM"), KeyEvent.VK_M);
		}
		return interactionPane;
	}

	/**
	 * This method initializes overviewPane	
	 * 	
	 * @return javax.swing.JTabbedPane	
	 */
	private JTabbedPane getOverviewPane() {
		if (overviewPane == null) {
			overviewPane = new JTabbedPane();
			overviewPane.addTab("Accounts", null, accountsPanel, null);
			overviewPane.setMnemonicAt(overviewPane.indexOfTab("Accounts"), KeyEvent.VK_A);
			overviewPane.addTab("Cards", null, cardsPanel, null);
			overviewPane.setMnemonicAt(overviewPane.indexOfTab("Cards"), KeyEvent.VK_C);
		}
		return overviewPane;
	}

	/**
	 * This method initializes quitButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getQuitButton() {
		if (quitButton == null) {
			quitButton = new JButton();
			quitButton.setText("Quit");
			quitButton.setMnemonic(KeyEvent.VK_Q);
			quitButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					getMainWindow().dispose();
					//getMainWindow().dispatchEvent(new WindowEvent(getMainWindow(), WindowEvent.WINDOW_CLOSING));
				}
			});
		}
		return quitButton;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				BankGui thisClass = new BankGui();
				thisClass.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				thisClass.setVisible(true);
			}
		});
	}

	/**
	 * This is the default constructor
	 */
	public BankGui() {
		super();

		atm = new ATM();

		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(447, 641);

		centralHostPanel = new CentralHostPanel(this, host, this);
		atmPanel = new ATMPanel(this, atm, host, this);
		accountsPanel = new AccountsPanel(host, this);
		cardsPanel = new CardsPanel();

		this.setContentPane(getMainPane());
		this.setTitle("Milliardenbank");
		updateTimeView();
	}

	private void updateTimeView() {
		int time = clock.getCurrentDate();
		dateLabel.setText(printDate(time));
		timeLabel.setText("(absolute time: " + time + ")");
	}

	/**
	 * This method initializes mainPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getMainPane() {
		if (mainPane == null) {
			GridBagConstraints quitButtonConstraints = new GridBagConstraints();
			quitButtonConstraints.gridx = 1;
			quitButtonConstraints.insets = new Insets(5, 5, 5, 10);
			quitButtonConstraints.gridy = 1;
			GridBagConstraints timePanelConstraints = new GridBagConstraints();
			timePanelConstraints.gridx = 0;
			timePanelConstraints.weightx = 1.0D;
			timePanelConstraints.fill = GridBagConstraints.HORIZONTAL;
			timePanelConstraints.insets = new Insets(5, 5, 5, 5);
			timePanelConstraints.gridy = 1;
			GridBagConstraints splitPaneConstraints = new GridBagConstraints();
			splitPaneConstraints.fill = GridBagConstraints.BOTH;
			splitPaneConstraints.weightx = 1.0;
			splitPaneConstraints.weighty = 1.0;
			splitPaneConstraints.gridx = 0;
			splitPaneConstraints.gridwidth = 2;
			splitPaneConstraints.gridy = 0;
			mainPane = new JPanel();
			mainPane.setLayout(new GridBagLayout());
			mainPane.add(getJSplitPane(), splitPaneConstraints);
			mainPane.add(getTimePanel(), timePanelConstraints);
			mainPane.add(getQuitButton(), quitButtonConstraints);
		}
		return mainPane;
	}


	/**
	 * @return the main window of the application
	 */
	private Window getMainWindow()
	{
		return this;
	}

	public void accountsChanged() {
		accountsPanel.updateAccounts();
	}

	public void addCard(BankCard card) {
		cardsPanel.addCard(card);
	}

	public BankCard selectCard() {
		return cardsPanel.selectCard(this);
	}

	public BankCard selectCard(Component parentComponent) {
		return cardsPanel.selectCard(parentComponent);
	}

	public void cardsChanged() {
		cardsPanel.updateCards();
	}

	public String printDate(int date) {
		return "day " + Clock.getDay(date) + ", month " + Clock.getMonth(date);
	}
}  //  @jve:decl-index=0:visual-constraint="10,10"
