package bankgui;

import bank.BankCard;

import java.awt.Component;


/**
 * Interface for basic actions and change notifications within the
 * bank model.
 */
interface BankModelInteraction {

	/**
	 * Add a new bank card to the collection of cards.
	 * 
	 * @param card the bank card to add
	 */
	void addCard (BankCard card);

	/**
	 * Let the user select one of the bank cards in a dialog.
	 * 
	 * @return the selected bank card, <code>null</code> if the user
	 * 			didn't select a card
	 */
	BankCard selectCard();

	/**
	 * Let the user select one of the bank cards in a dialog.
	 * 
	 * @param parentComponent the parent component for the selection dialog
	 * @return the selected bank card, <code>null</code> if the user
	 * 			didn't select a card
	 */
	BankCard selectCard(Component parentComponent);

	/**
	 * Notification that the set of accounts has changed (e.g. a new account
	 * has been created, the balance of an account has changed, ...).
	 */
	void accountsChanged();

	/**
	 * Notification that the set of bank cards has changed (e.g. a card 
	 * has become invalid). Doesn't have to be invoked if a new card is added
	 * using <code>addCard(BankCard)</code>.
	 */
	void cardsChanged();

	/**
	 * Print a given date to a string.
	 * 
	 * @param date the date as an absolute time, as returned by
	 * 				<code>Clock.getCurrentDate()</code>
	 * @return <code>date</code> in human readable form as a string
	 */
	String printDate (int date);
}
