package bankgui;

import java.awt.Component;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.KeyEvent;

import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JList;
import javax.swing.ListSelectionModel;
import javax.swing.JButton;

import java.util.Vector;

import bank.BankCard;


public class CardsPanel extends JPanel {

	private Vector bankCards = new Vector();

	private static final long serialVersionUID = 1L;
	private JScrollPane cardsScrollPane = null;
	private JList cardsJList = null;
	private JButton infoButton = null;

	/**
	 * This is the default constructor
	 */
	public CardsPanel() {
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		GridBagConstraints infoButtonConstraints = new GridBagConstraints();
		infoButtonConstraints.gridx = 0;
		infoButtonConstraints.insets = new Insets(5, 5, 5, 5);
		infoButtonConstraints.gridy = 1;
		GridBagConstraints cardsScrollPaneConstraints = new GridBagConstraints();
		cardsScrollPaneConstraints.fill = GridBagConstraints.BOTH;
		cardsScrollPaneConstraints.weighty = 1.0;
		cardsScrollPaneConstraints.gridx = 0;
		cardsScrollPaneConstraints.gridy = 0;
		cardsScrollPaneConstraints.weightx = 1.0;
		this.setSize(300, 200);
		this.setLayout(new GridBagLayout());
		this.add(getCardsScrollPane(), cardsScrollPaneConstraints);
		this.add(getInfoButton(), infoButtonConstraints);
	}

	/**
	 * This method initializes cardsScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getCardsScrollPane() {
		if (cardsScrollPane == null) {
			cardsScrollPane = new JScrollPane();
			cardsScrollPane.setViewportView(getCardsJList());
		}
		return cardsScrollPane;
	}

	/**
	 * This method initializes cardsJList	
	 * 	
	 * @return javax.swing.JList	
	 */
	private JList getCardsJList() {
		if (cardsJList == null) {
			cardsJList = new JList(bankCards);
			cardsJList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			cardsJList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
						public void valueChanged(javax.swing.event.ListSelectionEvent e) {
		                    if (cardsJList.isSelectionEmpty()) {
		                        infoButton.setEnabled(false);
		                    }
		                    else {
		                        infoButton.setEnabled(true);
		                    }
						}
					});
		}
		return cardsJList;
	}

	/**
	 * This method initializes infoButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getInfoButton() {
		if (infoButton == null) {
			infoButton = new JButton();
			infoButton.setText("Info ...");
			infoButton.setMnemonic(KeyEvent.VK_N);
			infoButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
                    if (cardsJList.isSelectionEmpty())
                        return;
                    BankCard card = (BankCard)cardsJList.getSelectedValue();
                    JOptionPane.showMessageDialog(getMainComponent(), card + "\n\n" +
                    		"Account number: " + card.getAccountNumber() + "\n" +
                            "Card is " + (card.cardIsInvalid() ? "invalid" : "valid") + ".",
                            "Card information", JOptionPane.PLAIN_MESSAGE);
				}
			});
			infoButton.setEnabled(false);
		}
		return infoButton;
	}

	/**
	 * Add a new bank card to the collection of cards.
	 * 
	 * @param card the bank card to add
	 */
	void addCard (BankCard card) {
		bankCards.add(card);
		cardsJList.updateUI();
	}

	/**
	 * Update the view of the bank cards (if the set of cards has changed). 
	 */
	void updateCards() {
		cardsJList.updateUI();
	}

	/**
	 * Let the user select one of the bank cards in a dialog.
	 * 
	 * @param parentComponent the parent component for the selection dialog
	 * @return the selected bank card, <code>null</code> if the user
	 * 			didn't select a card
	 */
	BankCard selectCard(Component parentComponent) {
		return (BankCard) JOptionPane.showInputDialog(parentComponent,
				"Please select a bank card:", "Card selection",
				JOptionPane.PLAIN_MESSAGE, null, bankCards.toArray(), null);
	}

	private Component getMainComponent() {
		return this;
	}
}
