package bankgui;

import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.KeyEvent;
import java.awt.GridBagConstraints;
import java.awt.Component;
import java.awt.Frame;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.BorderFactory;
import javax.swing.border.TitledBorder;
import javax.swing.JButton;

import bank.BankCard;
import bank.CentralHost;


public class CentralHostPanel extends JPanel {

	private CentralHost host;
	private BankModelInteraction modelInteraction;
	private Frame ownerFrame;

	private static final long serialVersionUID = 1L;
	private JPanel operationsPanel = null;
	private JButton createAccountButton = null;
	private JButton issueCardButton = null;
	private JButton reactivateCardButton = null;

	/**
	 * @param host the central host to be a GUI for
	 */
	public CentralHostPanel (Frame owner, CentralHost host, BankModelInteraction modelInteraction) {
		super();
		this.host = host;
		this.modelInteraction = modelInteraction;
		this.ownerFrame = owner;
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		GridBagConstraints spacerConstraints = new GridBagConstraints();
		spacerConstraints.gridx = 1;
		spacerConstraints.gridy = 0;
		spacerConstraints.weightx = 1.0D;
		GridBagConstraints operationsPanelConstraints = new GridBagConstraints();
		operationsPanelConstraints.gridx = 0;
		operationsPanelConstraints.fill = GridBagConstraints.VERTICAL;
		operationsPanelConstraints.weightx = 0.0D;
		operationsPanelConstraints.weighty = 1.0D;
		operationsPanelConstraints.gridy = 0;
		this.setLayout(new GridBagLayout());
		this.add(getOperationsPanel(), operationsPanelConstraints);
		this.add(new JLabel(), spacerConstraints);
	}

	/**
	 * This method initializes operationsPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getOperationsPanel() {
		if (operationsPanel == null) {
			GridBagConstraints spacerConstraints = new GridBagConstraints();
			spacerConstraints.gridx = 0;
			spacerConstraints.weighty = 1.0D;
			spacerConstraints.gridy = 3;
			GridBagConstraints createAccountButtonConstraints = new GridBagConstraints();
			createAccountButtonConstraints.gridx = 0;
			createAccountButtonConstraints.fill = GridBagConstraints.HORIZONTAL;
			createAccountButtonConstraints.insets = new Insets(5, 5, 5, 5);
			createAccountButtonConstraints.gridy = 0;
			GridBagConstraints reactivateCardButtonConstraints = new GridBagConstraints();
			reactivateCardButtonConstraints.gridx = 0;
			reactivateCardButtonConstraints.fill = GridBagConstraints.HORIZONTAL;
			reactivateCardButtonConstraints.insets = new Insets(5, 5, 5, 5);
			reactivateCardButtonConstraints.gridy = 2;
			GridBagConstraints issueCardButtonConstraints = new GridBagConstraints();
			issueCardButtonConstraints.gridx = 0;
			issueCardButtonConstraints.fill = GridBagConstraints.HORIZONTAL;
			issueCardButtonConstraints.insets = new Insets(5, 5, 5, 5);
			issueCardButtonConstraints.gridy = 1;
			operationsPanel = new JPanel();
			operationsPanel.setLayout(new GridBagLayout());
			operationsPanel.setBorder(BorderFactory.createTitledBorder(null, "Operations", TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.DEFAULT_POSITION, null, null));
			operationsPanel.add(getCreateAccountButton(), createAccountButtonConstraints);
			operationsPanel.add(getIssueCardButton(), issueCardButtonConstraints);
			operationsPanel.add(getReactivateCardButton(), reactivateCardButtonConstraints);
			operationsPanel.add(new JLabel(), spacerConstraints);
		}
		return operationsPanel;
	}

	/**
	 * This method initializes createAccountButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getCreateAccountButton() {
		if (createAccountButton == null) {
			createAccountButton = new JButton();
			createAccountButton.setText("Create Account");
			createAccountButton.setMnemonic(KeyEvent.VK_E);
			createAccountButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					String errorMessage = null;
					String accountNoString = JOptionPane.showInputDialog(getMainComponent(),
							"Account number (between 0 and " + (host.maxAccountNumber - 1) + "):",
                            "Create account", JOptionPane.PLAIN_MESSAGE);
					if (accountNoString == null)
						return;
					try {
						int accountNo = Integer.parseInt(accountNoString);
						if (accountNo < 0 || accountNo >= host.maxAccountNumber)
							errorMessage = "Account number must be between 0 and " + (host.maxAccountNumber - 1) + ".";
						else if (host.accountExists(accountNo))
							errorMessage = "Account no. " + accountNo + " already exists.";
						else
							host.createAccount(accountNo);
					} catch (NumberFormatException exc) {
						errorMessage = "Invalid input (" + exc.getMessage() + ").";
					}
					if (errorMessage != null)
						JOptionPane.showMessageDialog(getMainComponent(),
								errorMessage, "Error", JOptionPane.ERROR_MESSAGE);
					else
						modelInteraction.accountsChanged();
				}
			});
		}
		return createAccountButton;
	}

	/**
	 * This method initializes issueCardButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getIssueCardButton() {
		if (issueCardButton == null) {
			issueCardButton = new JButton();
			issueCardButton.setText("Issue Card");
			issueCardButton.setMnemonic(KeyEvent.VK_I);
			issueCardButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					int accountNumber, pin;
					String errorMessage = null;
					CreateCardDialog createCardDialog = new CreateCardDialog(ownerFrame);
					createCardDialog.setLocationRelativeTo(getMainComponent());
					createCardDialog.setVisible(true);
					if (createCardDialog.cancelled())
						return;
					try {
						accountNumber = createCardDialog.getAccountNumber();
						pin = createCardDialog.getPIN();
						if (accountNumber < 0 || accountNumber >= host.maxAccountNumber)
							errorMessage = "Account number must be between 0 and " + (host.maxAccountNumber - 1) + ".";
						else if (!host.accountExists(accountNumber))
							errorMessage = "Account no. " + accountNumber + " doesn't exist.";
						else
							modelInteraction.addCard(host.issueCard(accountNumber, pin));
					}
					catch (IllegalArgumentException exc) {
						errorMessage = exc.getMessage();
					}

					if (errorMessage != null)
						JOptionPane.showMessageDialog(getMainComponent(),
								errorMessage, "Error", JOptionPane.ERROR_MESSAGE);
				}
			});
		}
		return issueCardButton;
	}

	/**
	 * This method initializes reactivateCardButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getReactivateCardButton() {
		if (reactivateCardButton == null) {
			reactivateCardButton = new JButton();
			reactivateCardButton.setText("Reactivate Card");
			reactivateCardButton.setMnemonic(KeyEvent.VK_R);
			reactivateCardButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					BankCard card = modelInteraction.selectCard(getMainComponent());
					if (card == null)
						return;
					if (!card.cardIsInvalid())
						JOptionPane.showMessageDialog(getMainComponent(),
								"The selected bank card is not invalid.", "Error", JOptionPane.ERROR_MESSAGE);
					else {
						host.reactivateCard(card);
						modelInteraction.cardsChanged();
						JOptionPane.showMessageDialog(getMainComponent(),
								"The bank card has been reactivated.", "Card reactivated", JOptionPane.INFORMATION_MESSAGE);
					}
				}
			});
		}
		return reactivateCardButton;
	}

	private Component getMainComponent() {
		return this;
	}
}
