package bankgui;

import java.awt.Frame;
import java.awt.Window;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.FlowLayout;

import javax.swing.JPanel;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JButton;


public class TransferMoneyDialog extends JDialog {

	private static final long serialVersionUID = 1L;
	private JPanel mainPane = null;
	private JLabel accountNoLabel = null;
	private JTextField amountTextField = null;
	private JLabel amountLabel = null;
	private JTextField accountNoTextField = null;
	private JButton okButton = null;
	private boolean cancelled = false;
	private JButton cancelButton = null;
	private JPanel buttonPanel = null;

	/**
	 * @param owner
	 */
	public TransferMoneyDialog(Frame owner) {
		super(owner, "Transfer money", true);
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(269, 126);
		this.setContentPane(getMainPane());
		this.addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent e) {
				cancelled = true;
			}
		});
	}

	/**
	 * This method initializes mainPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getMainPane() {
		if (mainPane == null) {
			GridBagConstraints buttonPanelConstraints = new GridBagConstraints();
			buttonPanelConstraints.gridx = 0;
			buttonPanelConstraints.gridwidth = 2;
			buttonPanelConstraints.fill = GridBagConstraints.HORIZONTAL;
			buttonPanelConstraints.gridy = 3;
			GridBagConstraints accountNoTextFieldConstraints = new GridBagConstraints();
			accountNoTextFieldConstraints.fill = GridBagConstraints.VERTICAL;
			accountNoTextFieldConstraints.gridy = 0;
			accountNoTextFieldConstraints.weightx = 1.0;
			accountNoTextFieldConstraints.anchor = GridBagConstraints.WEST;
			accountNoTextFieldConstraints.insets = new Insets(0, 5, 0, 0);
			accountNoTextFieldConstraints.gridx = 1;
			GridBagConstraints amountLabelConstraints = new GridBagConstraints();
			amountLabelConstraints.gridx = 0;
			amountLabelConstraints.anchor = GridBagConstraints.EAST;
			amountLabelConstraints.insets = new Insets(0, 10, 0, 0);
			amountLabelConstraints.gridy = 2;
			amountLabel = new JLabel();
			amountLabel.setText("Amount:");
			GridBagConstraints amountTextFieldConstraints = new GridBagConstraints();
			amountTextFieldConstraints.fill = GridBagConstraints.VERTICAL;
			amountTextFieldConstraints.gridy = 2;
			amountTextFieldConstraints.weightx = 1.0;
			amountTextFieldConstraints.anchor = GridBagConstraints.WEST;
			amountTextFieldConstraints.insets = new Insets(0, 5, 0, 0);
			amountTextFieldConstraints.gridx = 1;
			GridBagConstraints accountNoLabelConstraints = new GridBagConstraints();
			accountNoLabelConstraints.gridx = 0;
			accountNoLabelConstraints.insets = new Insets(0, 10, 0, 0);
			accountNoLabelConstraints.anchor = GridBagConstraints.EAST;
			accountNoLabelConstraints.gridy = 0;
			accountNoLabel = new JLabel();
			accountNoLabel.setText("To account no.:");
			mainPane = new JPanel();
			mainPane.setLayout(new GridBagLayout());
			mainPane.add(accountNoLabel, accountNoLabelConstraints);
			mainPane.add(getAccountNoTextField(), accountNoTextFieldConstraints);
			mainPane.add(amountLabel, amountLabelConstraints);
			mainPane.add(getAmountTextField(), amountTextFieldConstraints);
			mainPane.add(getButtonPanel(), buttonPanelConstraints);
		}
		return mainPane;
	}

	/**
	 * This method initializes amountTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getAmountTextField() {
        if (amountTextField == null) {
            amountTextField = new JTextField();
            amountTextField.setColumns(10);
            amountTextField.addKeyListener(new java.awt.event.KeyAdapter() {   
				public void keyPressed(java.awt.event.KeyEvent e) {
					switch (e.getKeyCode()) {
						case java.awt.event.KeyEvent.VK_ESCAPE:
							cancelled = true;
							getMainWindow().dispose();
							break;
						case java.awt.event.KeyEvent.VK_ENTER:
							getMainWindow().dispose();
							break;
					}
				}
			});
        }
        return amountTextField;
	}

	/**
	 * This method initializes accountNoTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getAccountNoTextField() {
		if (accountNoTextField == null) {
            accountNoTextField = new JTextField();
			accountNoTextField.setColumns(6);
			accountNoTextField.addKeyListener(new java.awt.event.KeyAdapter() {   
				public void keyPressed(java.awt.event.KeyEvent e) {
					switch (e.getKeyCode()) {
						case java.awt.event.KeyEvent.VK_ESCAPE:
							cancelled = true;
							getMainWindow().dispose();
							break;
						case java.awt.event.KeyEvent.VK_ENTER:
							amountTextField.requestFocusInWindow();
							break;
					}
				}
			});
		}
		return accountNoTextField;
	}

	/**
	 * This method initializes okButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getOkButton() {
		if (okButton == null) {
			okButton = new JButton();
			okButton.setText("OK");
			okButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					getMainWindow().dispose();
				}
			});
		}
		return okButton;
	}

	/**
	 * @return the account number of the destination account
	 * @throws IllegalArgumentException if the user input has been invalid
	 */
	public int getDestinationAccountNumber() throws IllegalArgumentException {
		int accountNumber;
		try {
			accountNumber = Integer.parseInt(accountNoTextField.getText());
		}
		catch (NumberFormatException exc) {
			throw new IllegalArgumentException("Invalid input for account number: \"" + accountNoTextField.getText() + "\"");
		}
		return accountNumber;
	}

	/**
	 * @return the amount to transfer
	 * @throws IllegalArgumentException if the user input has been invalid
	 */
	public int getAmount() throws IllegalArgumentException {
		int amount;
		try {
			amount = Integer.parseInt(amountTextField.getText());
		}
		catch (NumberFormatException exc) {
			throw new IllegalArgumentException("Invalid input for amount: \"" + amountTextField.getText() + "\"");
		}
		return amount;
	}

	/**
	 * @return <code>true</code> iff the user cancelled the transfer
	 */
	public boolean cancelled() {
		return cancelled;
	}

	/**
	 * @return the main window of the application
	 */
	private Window getMainWindow()
	{
		return this;
	}

	/**
	 * This method initializes cancelButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText("Cancel");
			cancelButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					cancelled = true;
					getMainWindow().dispose();
				}
			});
		}
		return cancelButton;
	}

	/**
	 * This method initializes buttonPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getButtonPanel() {
		if (buttonPanel == null) {
			buttonPanel = new JPanel();
			buttonPanel.setLayout(new FlowLayout());
			buttonPanel.add(getOkButton(), null);
			buttonPanel.add(getCancelButton(), null);
		}
		return buttonPanel;
	}
}  //  @jve:decl-index=0:visual-constraint="10,10"
